@@ -1,7 +1,27 @@
+/*
+ * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ *
+ * This program is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free Software
+ * Foundation; either version 2 of the License, or (at your option) any later
+ * version.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
+
+/* $Id: nfst.c 3896 2012-10-10 12:19:26Z tovo $ */
+
 /**
  * Library.
  * Includes simple and fast computation of the NFST (direct problem)
- * authors: S. Klatt (c) 2004-2006
+ * author: S. Klatt 2004-2006
  */
 
 #include <stdio.h>
@@ -9,14 +29,11 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "util.h"
-#include "options.h"
-#include "window_defines.h"
-
+#include "nfft3util.h"
 #include "nfft3.h"
+#include "infft.h"
 
-
-/** 
+/**
  *  handy shortcuts
  **/
 #define NFST_DEFAULT_FLAGS   PRE_PHI_HUT|\
@@ -40,20 +57,20 @@
 
 
 #define MACRO_nfst_D_init_result_A        \
-  memset(g_hat, 0, nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));  
+  memset(g_hat, 0, nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
 #define MACRO_nfst_D_init_result_T        \
-  memset(f_hat, 0, ths->N_total * sizeof( double));  
+  memset(f_hat, 0, ths->N_total * sizeof( double));
 
 #define MACRO_nfst_B_init_result_A        \
-  memset(f, 0,     ths->M_total * sizeof( double));  
+  memset(f, 0,     ths->M_total * sizeof( double));
 #define MACRO_nfst_B_init_result_T        \
-  memset(g, 0, nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));  
+  memset(g, 0, nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
+
 
- 
 #define NFST_PRE_WINFUN( d)  ths->N[d] = 2 * ths->N[d];         \
                              ths->n[d] = nfst_fftw_2N( ths->n[d]);
 
-#define NFST_POST_WINFUN( d) ths->N[d] = ((int)(0.5 * ths->N[d]));   \
+#define NFST_POST_WINFUN( d) ths->N[d] = (LRINT(0.5 * ths->N[d]));   \
                              ths->n[d] = nfst_fftw_2N_rev( ths->n[d]);
 
 
@@ -71,12 +88,12 @@ double nfst_phi_hut( nfst_plan *ths, int k, int d)
 
 double nfst_phi( nfst_plan *ths, double x, int d)
 {
-  NFST_PRE_WINFUN( d); 
-  double phi_tmp = PHI( x, d); 
-  NFST_POST_WINFUN( d); 
- 
-  return phi_tmp; 
-} 
+  NFST_PRE_WINFUN( d);
+  double phi_tmp = PHI( x, d);
+  NFST_POST_WINFUN( d);
+
+  return phi_tmp;
+}
 
 int nfst_fftw_2N( int n)
 {
@@ -86,35 +103,35 @@ int nfst_fftw_2N( int n)
 int nfst_fftw_2N_rev( int n)
 {
   div_t n_div;
-  
+
   n_div = div(n, 2);
   return n_div.quot - 1;
 }
 
-#define MACRO_with_sin_vec     sin_vec[t][ka[t]] 
-#define MACRO_without_sin_vec  sin( 2.0 * PI * (ka[t]+1) * NODE(j,t)) 
- 
- 
-#define MACRO_with_PRE_PHI_HUT     ths->c_phi_inv[t][kg[t]]; 
+#define MACRO_with_sin_vec     sin_vec[t][ka[t]]
+#define MACRO_without_sin_vec  sin( 2.0 * PI * (ka[t]+1) * NODE(j,t))
+
+
+#define MACRO_with_PRE_PHI_HUT     ths->c_phi_inv[t][kg[t]];
 #define MACRO_compute_PHI_HUT_INV  (1.0 / (nfst_phi_hut( ths, kg[t]+1, t)))
 
-#define MACRO_with_PRE_PSI     ths->psi[(j * ths->d + t) * NFST_SUMMANDS + lc[t]];  
+#define MACRO_with_PRE_PSI     ths->psi[(j * ths->d + t) * NFST_SUMMANDS + lc[t]];
 #define MACRO_compute_PSI      \
-  nfst_phi( ths, NODE(j,t) - (( double)(lc[t] + lb[t])) / nfst_fftw_2N( ths->n[t]), t)  
+  nfst_phi( ths, NODE(j,t) - (( double)(lc[t] + lb[t])) / nfst_fftw_2N( ths->n[t]), t)
 
 
 
 /** direct computation of non equispaced sine transforms
- *  ndst_trafo,  ndst_adjoint_
+ *  nfst_trafo_direct,  nfst_adjoint_direct_
  *  require O(M N^d) arithemtical operations
  *
- * direct computation of the ndst_trafo, formula (1.1)
- * ndst_trafo:
- * for j=0,...,M-1                                                             
+ * direct computation of the nfst_trafo_direct, formula (1.1)
+ * nfst_trafo_direct:
+ * for j=0,...,M-1
  *  f[j] = sum_{k in I_N^d} f_hat[k] * sin(2 (pi) k x[j])
  *
- * direct computation of the ndft_adjoint, formula (1.2)
- * ndst_adjoint:
+ * direct computation of the nfft_adjoint_direct, formula (1.2)
+ * nfst_adjoint_direct:
  * for k in I_N^d
  *  f_hat[k] = sum_{j=0}^{M-1} f[j] * sin(2 (pi) k x[j])
  */
@@ -122,9 +139,9 @@ int nfst_fftw_2N_rev( int n)
 #define MACRO_ndst_malloc__sin_vec                                              \
                                                                                 \
   double **sin_vec;                                                             \
-  sin_vec = (double**)malloc( ths->d * sizeof( double*));                       \
+  sin_vec = (double**)nfft_malloc( ths->d * sizeof( double*));                       \
   for( t = 0; t < ths->d; t++)                                                  \
-    sin_vec[t] = (double*)malloc( ( ths->N[t] - 1) * sizeof( double));          \
+    sin_vec[t] = (double*)nfft_malloc( ( ths->N[t] - 1) * sizeof( double));          \
 
 
 
@@ -133,8 +150,8 @@ int nfst_fftw_2N_rev( int n)
 {                                                                               \
   /* free allocated memory */                                                   \
   for( t = 0; t < ths->d; t++)                                                  \
-    free( sin_vec[t]);                                                          \
-  free( sin_vec);                                                               \
+  nfft_free( sin_vec[t]);                                                          \
+  nfft_free( sin_vec);                                                               \
 }
 
 
@@ -195,7 +212,7 @@ int nfst_fftw_2N_rev( int n)
 
 /* slow (trafo) transform */
 #define MACRO_ndst( which_one)                                                  \
-  void ndst_ ## which_one ( nfst_plan *ths)                                     \
+  void nfst_ ## which_one ## _direct ( nfst_plan *ths)                                     \
   {                                                                             \
     int j, k, t, i;                                                             \
     int ka[ths->d];                                                             \
@@ -230,7 +247,7 @@ int nfst_fftw_2N_rev( int n)
         }                                                                       \
       else                                                                      \
       {                                                                         \
-        /* fast ndst_trafo */                                                   \
+        /* fast nfst_trafo_direct */                                                   \
         MACRO_ndst_malloc__sin_vec;                                             \
                                                                                 \
         for( j = 0; j < ths->M_total; j++)                                      \
@@ -250,7 +267,7 @@ int nfst_fftw_2N_rev( int n)
       }                                                                         \
   } /* ndst_{trafo, adjoint} */
 
- 
+
 MACRO_ndst(trafo)
 MACRO_ndst(adjoint)
 
@@ -262,7 +279,7 @@ MACRO_ndst(adjoint)
  *
  * fast computation of the nfst_trafo, formula (1.1)
  * nfst_trafo:
- * for j=0,...,M-1                                                             
+ * for j=0,...,M-1
  *  f[j] = sum_{k in I_N^d} f_hat[k] * sin(2 (pi) k x[j])
  *
  * direct computation of the nfst_adjoint, formula (1.2)
@@ -274,7 +291,7 @@ MACRO_ndst(adjoint)
 #define MACRO_nfst__lower_boundary( j,act_dim)                                  \
 {                                                                               \
   lb[(act_dim)] =                                                               \
-    ((int)(NODE((j),(act_dim)) * nfst_fftw_2N( ths->n[(act_dim)]))) - ths->m;   \
+    (LRINT(NODE((j),(act_dim)) * nfst_fftw_2N( ths->n[(act_dim)]))) - ths->m;   \
 }
 
 #define MACRO_nfst_D_compute_A                                                  \
@@ -310,7 +327,7 @@ MACRO_ndst(adjoint)
   }                                                                             \
 }
 
-  
+
 #define MACRO_update__c_phi_inv_k__lg_plain( which_one, which_phi)              \
 {                                                                               \
   for( t = i; t < ths->d; t++) {                                                \
@@ -327,7 +344,7 @@ MACRO_ndst(adjoint)
 
 
 #define MACRO_nfst_D(which_one)                                                 \
-inline void nfst_D_ ## which_one (nfst_plan *ths)                               \
+static inline void nfst_D_ ## which_one (nfst_plan *ths)                               \
 {                                                                               \
   int k_L;                              /**< plain index                */      \
                                                                                 \
@@ -372,7 +389,7 @@ inline void nfst_D_ ## which_one (nfst_plan *ths)
                                                                                 \
     } /* for(k_L) */                                                            \
 } /* nfst_D */
-  
+
 MACRO_nfst_D(A)
 MACRO_nfst_D(T)
 
@@ -384,7 +401,7 @@ MACRO_nfst_D(T)
 
 /** sub routines for the fast transforms
  *  matrix vector multiplication with \f$B, B^{\rm T}\f$
- */ 
+ */
 #define MACRO_nfst_B_PRE_FULL_PSI_compute_A                                     \
 {                                                                               \
   (*fj) += ths->psi[ix] * g[ths->psi_index_g[ix]];                              \
@@ -540,7 +557,7 @@ MACRO_nfst_D(T)
 
 
 #define MACRO_nfst_B( which_one)                                                \
-  inline void nfst_B_ ## which_one ( nfst_plan *ths)                            \
+  static inline void nfst_B_ ## which_one ( nfst_plan *ths)                            \
   { /* MACRO_nfst_B */                                                          \
     int lb[ths->d];                     /**< multi band with respect to x_j */  \
     int j, t, i;                        /**< index nodes, help vars         */  \
@@ -614,7 +631,7 @@ MACRO_nfst_D(T)
       } /* else(PRE_PSI) */                                                     \
     }/* else( PRE_PRE && FULL_PRE_PSI) */                                       \
 } /* nfst_B */
-  
+
 MACRO_nfst_B(A)
 MACRO_nfst_B(T)
 
@@ -623,25 +640,25 @@ MACRO_nfst_B(T)
 
 
 
-/** 
+/**
  * user routines
  *
  */
 void nfst_trafo( nfst_plan *ths)
 {
   /**
-   * use ths->my_fftw_r2r_plan 
+   * use ths->my_fftw_r2r_plan
    *
    */
   ths->g_hat = ths->g1;
   ths->g     = ths->g2;
 
- 
+
   /**
    * form \f$ \hat g_k = \frac{\hat f_k}{c_k\left(\phi\right)} \text{ for }
    * k \in I_N \f$
    *
-   */ 
+   */
   TIC(0)
   nfst_D_A( ths);
   TOC(0)
@@ -690,19 +707,19 @@ void nfst_adjoint( nfst_plan *ths)
   TIC(2)
   nfst_B_T( ths);
   TOC(2)
-      
+
 
   /**
    * compute by d-variate discrete cosine transform
    * \f$ \hat g_k = \sum_{l \in I_n} g_l {\rm e}^{-2\pi {\rm i} \frac{kl}{n}}
    * \text{ for }  k \in I_N\f$
    *
-   */ 
+   */
   TIC(1)
   fftw_execute( ths->my_fftw_r2r_plan);
   TOC(1)
 
-  
+
   /**
    * form \f$ \hat f_k = \frac{\hat g_k}{c_k\left(\phi\right)} \text{ for }
    * k \in I_N \f$
@@ -717,7 +734,7 @@ void nfst_adjoint( nfst_plan *ths)
 
 
 /**
- * initialization of direct transform 
+ * initialization of direct transform
  *
  */
 void nfst_precompute_phi_hut( nfst_plan *ths)
@@ -725,12 +742,12 @@ void nfst_precompute_phi_hut( nfst_plan *ths)
   int kg[ths->d];                      /**< index over all frequencies       */
   int t;                               /**< index over all dimensions        */
 
-  ths->c_phi_inv = (double**)fftw_malloc( ths->d * sizeof( double*));
+  ths->c_phi_inv = (double**)nfft_malloc( ths->d * sizeof( double*));
 
   for( t = 0; t < ths->d; t++)
   {
-    ths->c_phi_inv[t] = (double*)fftw_malloc( ( ths->N[t] - 1) * sizeof( double));
-    
+    ths->c_phi_inv[t] = (double*)nfft_malloc( ( ths->N[t] - 1) * sizeof( double));
+
     for( kg[t] = 0; kg[t] < ths->N[t] - 1; kg[t]++)
     {
       ths->c_phi_inv[t][kg[t]] = MACRO_compute_PHI_HUT_INV;
@@ -746,7 +763,7 @@ void nfst_precompute_psi( nfst_plan *ths)
   int j;                                /**< index over all nodes             */
   int lc[ths->d];                       /**< index 0<=lj<u+o+1                */
   int lb[ths->d];                       /**< depends on x_j                   */
-  
+
   for (t = 0; t < ths->d; t++)
   {
     for(j = 0; j < ths->M_total; j++)
@@ -780,7 +797,7 @@ void nfst_full_psi(nfst_plan *ths, double eps)
   int lg[ths->d];
   int lprod;                            /**< 'bandwidth' of matrix B          */
   int lb[ths->d];                       /**< depends on x_j                   */
-  
+
   double phi_tilde[ths->d+1];
 
   int *index_g, *index_f;
@@ -793,20 +810,20 @@ void nfst_full_psi(nfst_plan *ths, double eps)
   if(ths->nfst_flags & PRE_PSI)
   {
     size_psi = ths->M_total;
-    index_f  =    (int*)malloc( ths->M_total  * sizeof( int));
-    index_g  =    (int*)malloc( size_psi * sizeof( int));
-    new_psi  = (double*)malloc( size_psi * sizeof( double));
-    
+    index_f  =    (int*)nfft_malloc( ths->M_total  * sizeof( int));
+    index_g  =    (int*)nfft_malloc( size_psi * sizeof( int));
+    new_psi  = (double*)nfft_malloc( size_psi * sizeof( double));
+
     for( t = 0,lprod = 1; t < ths->d; t++)
     {
       lprod *= NFST_SUMMANDS;
       eps *= PHI( 0, t);
     }
-    
+
     for( ix = 0, ix_old = 0, j = 0; j < ths->M_total; j++)
     {
       MACRO_init_lb_lg_lc_phi_tilde_lg_plain( with_PRE_PSI);
-          
+
       for( l_L = 0; l_L < lprod; l_L++)
       {
         MACRO_update__phi_tilde__lg_plain( with_PRE_PSI);
@@ -815,7 +832,7 @@ void nfst_full_psi(nfst_plan *ths, double eps)
         {
           index_g[ix] =  lg_plain[ths->d];
           new_psi[ix] = phi_tilde[ths->d];
-  
+
           ix++;
           if( ix == size_psi)
           {
@@ -827,23 +844,23 @@ void nfst_full_psi(nfst_plan *ths, double eps)
         MACRO_count__lg_lc;
 
       } /* for(l_L) */
-      
+
       index_f[j] = ix - ix_old;
       ix_old     = ix;
-      
+
     } /* for(j) */
-    
-    free( ths->psi);
+
+    nfft_free( ths->psi);
 
     size_psi      = ix;
     ths->size_psi = size_psi;
     index_g       = (int*)realloc( index_g, size_psi * sizeof( int));
     new_psi       = (double*)realloc( new_psi, size_psi * sizeof( double));
-    
+
     ths->psi         = new_psi;
-    ths->psi_index_g = index_g; 
+    ths->psi_index_g = index_g;
     ths->psi_index_f = index_f;
-    
+
   } /* if(PRE_PSI) */
 } /* nfst_full_psi */
 
@@ -856,14 +873,14 @@ void nfst_init_help( nfst_plan *ths)
 
   ths->N_total = nfst_prod_minus_a_int( ths->N, 1, ths->d);
 
-  ths->sigma   = (double*)fftw_malloc( ths->d * sizeof( double));
+  ths->sigma   = (double*)nfft_malloc( ths->d * sizeof( double));
 
-  for( t = 0; t < ths->d; t++) 
+  for( t = 0; t < ths->d; t++)
     /* FIXME: n/N or (n+1)/N */
     ths->sigma[t] = ((double)ths->n[t] + 1) / ths->N[t];
 
   /* assign r2r transform kinds for each dimension */
-  ths->r2r_kind = (fftw_r2r_kind*) fftw_malloc ( ths->d * sizeof( fftw_r2r_kind));
+  ths->r2r_kind = (fftw_r2r_kind*) nfft_malloc ( ths->d * sizeof( fftw_r2r_kind));
   for (t = 0; t < ths->d; t++)
     ths->r2r_kind[t] = FFTW_RODFT00;
 
@@ -871,13 +888,13 @@ void nfst_init_help( nfst_plan *ths)
   WINDOW_HELP_INIT;
 
   if(ths->nfst_flags & MALLOC_X)
-    ths->x = (double*)fftw_malloc( ths->d * ths->M_total * sizeof( double));
+    ths->x = (double*)nfft_malloc( ths->d * ths->M_total * sizeof( double));
 
   if(ths->nfst_flags & MALLOC_F_HAT)
-    ths->f_hat = (double*)fftw_malloc( ths->N_total * sizeof( double));
+    ths->f_hat = (double*)nfft_malloc( ths->N_total * sizeof( double));
 
   if(ths->nfst_flags & MALLOC_F)
-    ths->f = (double*)fftw_malloc( ths->M_total * sizeof( double));
+    ths->f = (double*)nfft_malloc( ths->M_total * sizeof( double));
 
   if(ths->nfst_flags & PRE_PHI_HUT)
     nfst_precompute_phi_hut( ths);
@@ -886,28 +903,31 @@ void nfst_init_help( nfst_plan *ths)
   if(ths->nfst_flags & PRE_PSI)
   {
     ths->psi =
-      (double*)malloc( ths->M_total * ths->d * NFST_SUMMANDS * sizeof( double));
+      (double*)nfft_malloc( ths->M_total * ths->d * NFST_SUMMANDS * sizeof( double));
 
     /**
-     * set default for full_psi_eps 
+     * set default for full_psi_eps
      **/
     ths->nfst_full_psi_eps = pow(10, -10);
   }
-  
+
   if(ths->nfst_flags & FFTW_INIT)
   {
       ths->g1 =
-        (double*)fftw_malloc( nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
+        (double*)nfft_malloc( nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
 
       if(ths->nfst_flags & FFT_OUT_OF_PLACE)
         ths->g2 =
-          (double*)fftw_malloc( nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
+          (double*)nfft_malloc( nfst_prod_minus_a_int( ths->n, 0, ths->d) * sizeof( double));
       else
         ths->g2 = ths->g1;
 
-      ths->my_fftw_r2r_plan = 
+      ths->my_fftw_r2r_plan =
         fftw_plan_r2r( ths->d, ths->n, ths->g1, ths->g2, ths->r2r_kind, ths->fftw_flags);
   }
+
+  ths->mv_trafo = (void (*) (void* ))nfst_trafo;
+  ths->mv_adjoint = (void (*) (void* ))nfst_adjoint;
 }
 
 void nfst_init( nfst_plan *ths, int d, int *N, int M_total)
@@ -916,24 +936,27 @@ void nfst_init( nfst_plan *ths, int d, int *N, int M_total)
 
   ths->d = d;
 
-  ths->N      = (int*)fftw_malloc( ths->d * sizeof( int));
+  ths->N      = (int*)nfft_malloc( ths->d * sizeof( int));
 
   for(t = 0;t < d; t++)
     ths->N[t] = N[t];
 
-  ths->n      = (int*)fftw_malloc( ths->d * sizeof( int));
+  ths->n      = (int*)nfft_malloc( ths->d * sizeof( int));
 
   for( t = 0; t < d; t++)
-    ths->n[t] = 2 * next_power_of_2( ths->N[t]) - 1;
+    ths->n[t] = 2 * X(next_power_of_2)( ths->N[t]) - 1;
 
   ths->M_total = M_total;
 
+/* Was soll dieser Ausdruck machen? Es handelt sich um eine Ganzzahl!
+
   WINDOW_HELP_ESTIMATE_m;
+*/  
 
   ths->nfst_flags = NFST_DEFAULT_FLAGS;
   ths->fftw_flags = FFTW_DEFAULT_FLAGS;
 
-  nfst_init_help( ths);    
+  nfst_init_help( ths);
 }
 
 
@@ -941,8 +964,8 @@ void nfst_init_m( nfst_plan *ths, int d, int *N, int M_total, int m)
 {
   int t, n[d];
 
-  for( t = 0; t < d; t++) 
-    n[t] = nfst_fftw_2N( next_power_of_2( N[t])); 
+  for( t = 0; t < d; t++)
+    n[t] = nfst_fftw_2N( X(next_power_of_2)( N[t]));
 
   nfst_init_guru( ths, d, N, M_total, n, m, NFST_DEFAULT_FLAGS, FFTW_DEFAULT_FLAGS);
 }
@@ -957,12 +980,12 @@ void nfst_init_guru( nfst_plan *ths, int d, int *N,
   ths->d = d;
   ths->M_total = M_total;
 
-  ths->N      = (int*)fftw_malloc( ths->d * sizeof( int));
+  ths->N      = (int*)nfft_malloc( ths->d * sizeof( int));
 
   for( t = 0; t < d; t++)
     ths->N[t]      = N[t];
 
-  ths->n      = (int*)fftw_malloc( ths->d * sizeof( int));
+  ths->n      = (int*)nfft_malloc( ths->d * sizeof( int));
 
   for( t = 0; t < d; t++)
     ths->n[t]      = n[t];
@@ -972,7 +995,7 @@ void nfst_init_guru( nfst_plan *ths, int d, int *N,
   ths->nfst_flags = nfst_flags;
   ths->fftw_flags = fftw_flags;
 
-  nfst_init_help( ths);  
+  nfst_init_help( ths);
 }
 
 
@@ -1010,11 +1033,11 @@ void nfst_finalize( nfst_plan *ths)
   if( ths->nfst_flags & FFTW_INIT)
   {
     fftw_destroy_plan( ths->my_fftw_r2r_plan);
-      
+
     if( ths->nfst_flags & FFT_OUT_OF_PLACE)
-      fftw_free( ths->g2);
-      
-    fftw_free( ths->g1);
+      nfft_free( ths->g2);
+
+    nfft_free( ths->g1);
   }
 
   /* NO FFTW_FREE HERE */
@@ -1022,33 +1045,34 @@ void nfst_finalize( nfst_plan *ths)
   {
     if( ths->nfst_flags & PRE_FULL_PSI)
     {
-      free( ths->psi_index_g);
-      free( ths->psi_index_f);
+      nfft_free( ths->psi_index_g);
+      nfft_free( ths->psi_index_f);
     }
-    
-    free( ths->psi);
+
+    nfft_free( ths->psi);
   }
 
   if( ths->nfst_flags & PRE_PHI_HUT) {
     for( t = 0; t < ths->d; t++)
-      fftw_free( ths->c_phi_inv[t]);
-    fftw_free( ths->c_phi_inv);
+      nfft_free( ths->c_phi_inv[t]);
+    nfft_free( ths->c_phi_inv);
   }
 
   if( ths->nfst_flags & MALLOC_F)
-    fftw_free( ths->f);
+    nfft_free( ths->f);
 
   if( ths->nfst_flags & MALLOC_F_HAT)
-    fftw_free( ths->f_hat);
-  
+    nfft_free( ths->f_hat);
+
   if( ths->nfst_flags & MALLOC_X)
-    fftw_free( ths->x);
- 
+    nfft_free( ths->x);
+
   WINDOW_HELP_FINALIZE;
 
-  fftw_free( ths->N);
-  fftw_free( ths->n);
-  fftw_free( ths->sigma);
+  nfft_free( ths->N);
+  nfft_free( ths->n);
+  nfft_free( ths->sigma);
 
+  nfft_free(ths->r2r_kind);
 } /* nfst_finalize */
 
